unit RVReportListViewQueryProcessor;

interface

uses
  SysUtils, ComCtrls, Classes,
  RVTypes, RVItem, RVStrFuncs,
  RVReportDataProvider;

type

  TRVReportListViewQueryProcessor = class (TRVReportQueryProcessor)
  private
    FRecordIndex: Integer;
    FListView: TListView;
    procedure CheckFieldIndex(Index: Integer);
    procedure CheckRecordIndex;
  public
    function Execute: Boolean; override;
    function GetRecordCount: Integer; override;
    function MoveToFirstRecord: Boolean; override;
    function MoveToNextRecord: Boolean; override;
    function MoveBy(Distance: Cardinal): Boolean; override;
    function GetField(const FieldName: TRVUnicodeString): TRVReportField;
      overload; override;
    function GetField(Index: Integer): TRVReportField; overload; override;
    function GetFieldType(Field: TRVReportField): TRVReportFieldType; override;
    function GetFieldName(Field: TRVReportField): TRVUnicodeString; override;
    function GetAsString(Field: TRVReportField): TRVUnicodeString; override;
    function GetFieldCount: Integer; override;
    constructor Create(AListView: TListView);
    function IsFieldEmpty(Field: TRVReportField): Boolean; override;
  end;

implementation

const
  ErrInvalidField = 'Invalid field for a list view query processor';
  ErrInvalidRecordIndex = 'Invalid record index';

{======================= TRVReportListViewQueryProcessor ======================}
constructor TRVReportListViewQueryProcessor.Create(AListView: TListView);
begin
  inherited Create;
  FListView := AListView;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.Execute: Boolean;
begin
  Result := MoveToFirstRecord;
end;
{------------------------------------------------------------------------------}
procedure TRVReportListViewQueryProcessor.CheckRecordIndex;
begin
  if (FRecordIndex < 0) or (FRecordIndex >= FListView.Items.Count) then
    raise ERichViewError.Create(ErrInvalidRecordIndex);
end;
{------------------------------------------------------------------------------}
procedure TRVReportListViewQueryProcessor.CheckFieldIndex(Index: Integer);
begin
  if (Index < 1) or (Index > FListView.Columns.Count) then
    raise ERichViewError.Create(ErrInvalidField);
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetAsString(
  Field: TRVReportField): TRVUnicodeString;
begin
  CheckFieldIndex(Integer(Field));
  CheckRecordIndex;
  if Integer(Field) = 1 then
    // {1} returns Items[].Caption
    Result := FListView.Items[FRecordIndex].Caption
  else if Integer(Field) - 2 < FListView.Items[FRecordIndex].SubItems.Count then
    // {2} ... return Items[].SubItems[]
    Result := FListView.Items[FRecordIndex].SubItems[Integer(Field) - 2]
  else
    Result := '';
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetFieldCount: Integer;
begin
  Result := FListView.Columns.Count;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetField(
  const FieldName: TRVUnicodeString): TRVReportField;
begin
  // using field indexes + 1 as field identifiers and names
  Result := TRVReportField(RVStrToIntDefW(FieldName, -1));
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetField(
  Index: Integer): TRVReportField;
begin
  // using field indexes + 1 as field identifiers
  Result := TRVReportField(Index + 1);
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetFieldName(
  Field: TRVReportField): TRVUnicodeString;
begin
  CheckFieldIndex(Integer(Field));
  Result := RVIntToStrW(Integer(Field));
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetFieldType(
  Field: TRVReportField): TRVReportFieldType;
begin
  CheckFieldIndex(Integer(Field));
  Result := rvrftText;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.GetRecordCount: Integer;
begin
  Result := FListView.Items.Count;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.MoveBy(
  Distance: Cardinal): Boolean;
begin
  inc(FRecordIndex, Distance);
  Result := FRecordIndex <= FListView.Items.Count;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.MoveToFirstRecord: Boolean;
begin
  FRecordIndex := 0;
  Result := True;
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.MoveToNextRecord: Boolean;
begin
  Result := FRecordIndex < FListView.Items.Count;
  inc(FRecordIndex);
end;
{------------------------------------------------------------------------------}
function TRVReportListViewQueryProcessor.IsFieldEmpty(
  Field: TRVReportField): Boolean;
begin
  CheckFieldIndex(Integer(Field));
  CheckRecordIndex;
  Result := GetAsString(Field) <> '';
end;

end.
